using cuqmbr.TravelGuide.Application.Common.Authorization;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Services;
using cuqmbr.TravelGuide.Application.Common.Models;
using MediatR.Behaviors.Authorization;

namespace cuqmbr.TravelGuide.Application.Aircrafts.Commands.DeleteAircraft;

public class DeleteAircraftCommandAuthorizer :
    AbstractRequestAuthorizer<DeleteAircraftCommand>
{
    private readonly SessionUserService _sessionUserService;

    public DeleteAircraftCommandAuthorizer(SessionUserService sessionUserService)
    {
        _sessionUserService = sessionUserService;
    }

    public override void BuildPolicy(DeleteAircraftCommand request)
    {
        UseRequirement(new MustBeAuthenticatedRequirement
        {
            IsAuthenticated= _sessionUserService.IsAuthenticated
        });

        UseRequirement(new MustBeInRolesRequirement
        {
            RequiredRoles = [IdentityRole.Administrator],
            UserRoles = _sessionUserService.Roles
        });
    }
}
