using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Application.Common.Extensions;

namespace cuqmbr.TravelGuide.Application.Buses.Queries.GetBusesPage;

public class GetBusesPageQueryHandler :
    IRequestHandler<GetBusesPageQuery, PaginatedList<BusDto>>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetBusesPageQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<PaginatedList<BusDto>> Handle(
        GetBusesPageQuery request,
        CancellationToken cancellationToken)
    {
        var paginatedList = await _unitOfWork.BusRepository.GetPageAsync(
            e =>
                (e.Number.ToLower().Contains(request.Search.ToLower()) ||
                e.Model.ToLower().Contains(request.Search.ToLower())) &&
                (request.CompanyGuid != null
                    ? e.Company.Guid == request.CompanyGuid
                    : true) &&
                (request.CapacityGreaterThanOrEqualTo != null
                    ? e.Capacity >= request.CapacityGreaterThanOrEqualTo
                    : true) &&
                (request.CapacityLessThanOrEqualTo != null
                    ? e.Capacity <= request.CapacityLessThanOrEqualTo
                    : true),
            e => e.Company,
            request.PageNumber, request.PageSize,
            cancellationToken);

        var mappedItems = _mapper
            .ProjectTo<BusDto>(paginatedList.Items.AsQueryable());

        mappedItems = QueryableExtension<BusDto>
            .ApplySort(mappedItems, request.Sort);

        _unitOfWork.Dispose();

        return new PaginatedList<BusDto>(
            mappedItems.ToList(),
            paginatedList.TotalCount, request.PageNumber,
            request.PageSize);
    }
}
