using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using cuqmbr.TravelGuide.Domain.Entities;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Exceptions;

namespace cuqmbr.TravelGuide.Application.Companies.Commands.AddCompany;

public class AddCompanyCommandHandler : 
    IRequestHandler<AddCompanyCommand, CompanyDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public AddCompanyCommandHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<CompanyDto> Handle(
        AddCompanyCommand request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.CompanyRepository.GetOneAsync(
            e => e.Name == request.Name, cancellationToken);

        if (entity != null)
        {
            throw new DuplicateEntityException(
                "Company with given name already exists.");
        }

        entity = new Company()
        {
            Name = request.Name,
            LegalAddress = request.LegalAddress,
            ContactEmail = request.ContactEmail,
            ContactPhoneNumber = request.ContactPhoneNumber
        };

        entity = await _unitOfWork.CompanyRepository.AddOneAsync(
            entity, cancellationToken);

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();

        return _mapper.Map<CompanyDto>(entity);
    }
}
