using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Application.Common.Extensions;

namespace cuqmbr.TravelGuide.Application.Companies.Queries.GetCompaniesPage;

public class GetCompaniesPageQueryHandler :
    IRequestHandler<GetCompaniesPageQuery, PaginatedList<CompanyDto>>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetCompaniesPageQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<PaginatedList<CompanyDto>> Handle(
        GetCompaniesPageQuery request,
        CancellationToken cancellationToken)
    {
        var paginatedList = await _unitOfWork.CompanyRepository.GetPageAsync(
            e =>
                e.Name.ToLower().Contains(request.Search.ToLower()) ||
                e.LegalAddress.ToLower().Contains(request.Search.ToLower()) ||
                e.ContactEmail.ToLower().Contains(request.Search.ToLower()) ||
                e.ContactPhoneNumber.ToLower().Contains(request.Search.ToLower()),
            request.PageNumber, request.PageSize,
            cancellationToken);

        var mappedItems = _mapper
            .ProjectTo<CompanyDto>(paginatedList.Items.AsQueryable());

        mappedItems = QueryableExtension<CompanyDto>
            .ApplySort(mappedItems, request.Sort);

        _unitOfWork.Dispose();

        return new PaginatedList<CompanyDto>(
            mappedItems.ToList(),
            paginatedList.TotalCount, request.PageNumber,
            request.PageSize);
    }
}
