using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;
using AutoMapper;

namespace cuqmbr.TravelGuide.Application.Countries.Queries.GetCountry;

public class GetCountryQueryHandler :
    IRequestHandler<GetCountryQuery, CountryDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetCountryQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<CountryDto> Handle(
        GetCountryQuery request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.CountryRepository.GetOneAsync(
            e => e.Guid == request.Guid, cancellationToken);

        _unitOfWork.Dispose();

        if (entity == null)
        {
            throw new NotFoundException();
        }

        return _mapper.Map<CountryDto>(entity);
    }
}
