using cuqmbr.TravelGuide.Application.Common.Mappings;
using cuqmbr.TravelGuide.Domain.Entities;

namespace cuqmbr.TravelGuide.Application.TicketGroups;

public sealed class TicketDto : IMapFrom<Ticket>
{
    public Guid Uuid { get; set; }

    public Guid DepartureRouteAddressUuid { get; set; }

    public Guid ArrivalRouteAddressUuid { get; set; }

    public TicketAddressDto DepartureAddress { get; set; }

    public TicketAddressDto ArrivalAddress { get; set; }

    public short Order { get; set; }

    public Guid VehicleEnrollmentUuid { get; set; }

    // TODO: Add VehicleEnrollment model

    public string Currency { get; set; }

    public decimal Cost { get; set; }

    public void Mapping(MappingProfile profile)
    {
        profile.CreateMap<Ticket, TicketDto>()
            .ForMember(
                d => d.Uuid,
                opt => opt.MapFrom(s => s.Guid))
            .ForMember(
                d => d.DepartureRouteAddressUuid,
                opt => opt.MapFrom(s => s.DepartureRouteAddress.Guid))
            .ForMember(
                d => d.ArrivalRouteAddressUuid,
                opt => opt.MapFrom(s => s.ArrivalRouteAddress.Guid))
            .ForMember(
                d => d.DepartureAddress,
                opt => opt.MapFrom(s => s.DepartureRouteAddress.Address))
            .ForMember(
                d => d.ArrivalAddress,
                opt => opt.MapFrom(s => s.ArrivalRouteAddress.Address))
            .ForMember(
                d => d.VehicleEnrollmentUuid,
                opt => opt.MapFrom(s => s.VehicleEnrollment.Guid));
    }
}
