using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;
using AutoMapper;

namespace cuqmbr.TravelGuide.Application.Trains.Queries.GetTrain;

public class GetTrainQueryHandler :
    IRequestHandler<GetTrainQuery, TrainDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetTrainQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<TrainDto> Handle(
        GetTrainQuery request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.TrainRepository.GetOneAsync(
            e => e.Guid == request.Guid, e => e.Company,
            cancellationToken);

        _unitOfWork.Dispose();

        if (entity == null)
        {
            throw new NotFoundException();
        }

        return _mapper.Map<TrainDto>(entity);
    }
}
