using cuqmbr.TravelGuide.Application.Common.Interfaces.Services;
using cuqmbr.TravelGuide.Domain.Enums;
using FluentValidation;
using Microsoft.Extensions.Localization;

namespace cuqmbr.TravelGuide.Application
    .VehicleEnrollmentSearch.Queries.SearchShortest;

public class SearchShortestQueryValidator :
    AbstractValidator<SearchShortestQuery>
{
    public SearchShortestQueryValidator(
        IStringLocalizer localizer,
        SessionCultureService cultureService)
    {
        RuleFor(v => v.DepartureAddressGuid)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"]);

        RuleFor(v => v.ArrivalAddressGuid)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"]);

        RuleFor(v => v.DepartureDate)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"])
            .GreaterThanOrEqualTo(DateOnly.FromDateTime(DateTime.UtcNow))
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.GreaterThanOrEqualTo"],
                    DateOnly.FromDateTime(DateTime.UtcNow)));

        RuleForEach(v => v.VehicleTypes)
            .Must((v, vt) => VehicleType.Enumerations.ContainsValue(vt))
            .WithMessage(
                String.Format(
                    localizer["FluentValidation.MustBeInEnum"],
                    String.Join(
                        ", ",
                        VehicleType.Enumerations.Values.Select(e => e.Name))));
    }
}
