using cuqmbr.TravelGuide.Application.Common.Interfaces.Services;
using cuqmbr.TravelGuide.Domain.Enums;
using FluentValidation;
using Microsoft.Extensions.Localization;

namespace cuqmbr.TravelGuide.Application.VehicleEnrollments
    .Commands.AddVehicleEnrollment;

public class AddVehicleEnrollmentCommandValidator :
    AbstractValidator<AddVehicleEnrollmentCommand>
{
    public AddVehicleEnrollmentCommandValidator(
        IStringLocalizer localizer,
        SessionCultureService cultureService,
        SessionTimeZoneService timeZoneService)
    {
        RuleFor(v => v.DepartureTime)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"])
            .Must(dt => dt >= DateTimeOffset.Now)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.DateTimeOffset.GreaterThanOrEqualTo"],
                    DateTimeOffset.Now.ToOffset(timeZoneService.TimeZone.BaseUtcOffset)));

        RuleFor(v => v.Currency)
            .Must(c => Currency.Enumerations.ContainsValue(c))
            .WithMessage(
                String.Format(
                    localizer["FluentValidation.MustBeInEnum"],
                    String.Join(
                        ", ",
                        Currency.Enumerations.Values.Select(e => e.Name))));

        RuleFor(v => v.RouteAddressDetails.Count)
            .GreaterThanOrEqualTo(2)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.GreaterThanOrEqualTo"],
                    2));

        RuleFor(v => v.RouteAddressDetails)
            .Must(v => v.All(rad => rad.RouteAddressGuid != Guid.Empty))
            .WithMessage(localizer["FluentValidation.NotEmpty"])
            .Must(v => v.All(rad => rad.TimeToNextAddress >= TimeSpan.Zero))
            .WithMessage(localizer["VehicleEnrollments.NegativeTime"])
            .Must(v => v.All(rad => rad.CostToNextAddress >= 0))
            .WithMessage(localizer["VehicleEnrollments.NegativeCost"])
            .Must(v => v.All(rad => rad.CurrentAddressStopTime >= TimeSpan.Zero))
            .WithMessage(localizer["VehicleEnrollments.NegativeTime"]);


        RuleFor(v => v.VehicleGuid)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"]);

        RuleFor(v => v.RouteGuid)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"]);
    }
}
