using cuqmbr.TravelGuide.Application.Common.Authorization;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Services;
using cuqmbr.TravelGuide.Application.Common.Models;
using MediatR.Behaviors.Authorization;

namespace cuqmbr.TravelGuide.Application.VehicleEnrollments
    .Queries.GetVehicleEnrollment;

public class GetVehicleEnrollmentQueryAuthorizer :
    AbstractRequestAuthorizer<GetVehicleEnrollmentQuery>
{
    private readonly SessionUserService _sessionUserService;

    public GetVehicleEnrollmentQueryAuthorizer(SessionUserService sessionUserService)
    {
        _sessionUserService = sessionUserService;
    }

    public override void BuildPolicy(GetVehicleEnrollmentQuery request)
    {
        UseRequirement(new MustBeAuthenticatedRequirement
        {
            IsAuthenticated= _sessionUserService.IsAuthenticated
        });

        UseRequirement(new MustBeInRolesRequirement
        {
            RequiredRoles = [IdentityRole.Administrator],
            UserRoles = _sessionUserService.Roles
        });
    }
}
