using cuqmbr.TravelGuide.Application.Common.Mappings;
using cuqmbr.TravelGuide.Application.Common.Mappings.Resolvers;
using cuqmbr.TravelGuide.Domain.Entities;

namespace cuqmbr.TravelGuide.Application.VehicleEnrollments;

public sealed class VehicleEnrollmentDto : IMapFrom<VehicleEnrollment>
{
    public Guid Uuid { get; set; }

    public DateTimeOffset DepartureTime { get; set; }

    public DateTimeOffset ArrivalTime =>
        DepartureTime + 
        RouteAddressDetails.Aggregate(TimeSpan.Zero, (sum, next) =>
            sum + next.TimeToNextAddress + next.CurrentAddressStopTime);

    public TimeSpan TravelTime =>
        RouteAddressDetails.Aggregate(TimeSpan.Zero, (sum, next) =>
            sum + next.TimeToNextAddress + next.CurrentAddressStopTime);

    public TimeSpan TimeMoving =>
        RouteAddressDetails.Aggregate(TimeSpan.Zero, (sum, next) =>
            sum + next.TimeToNextAddress);

    public TimeSpan TimeInStops =>
        RouteAddressDetails.Aggregate(TimeSpan.Zero, (sum, next) =>
            sum + next.CurrentAddressStopTime);

    public decimal TotalCost =>
        RouteAddressDetails.Aggregate((decimal)0, (sum, next) =>
            sum + next.CostToNextAddress);

    public string Currency { get; set; }

    public ICollection<RouteAddressDetailDto> RouteAddressDetails { get; set; }

    public void Mapping(MappingProfile profile)
    {
        profile.CreateMap<VehicleEnrollment, VehicleEnrollmentDto>()
            .ForMember(
                d => d.Uuid,
                opt => opt.MapFrom(s => s.Guid))
            .ForMember(
                d => d.DepartureTime,
                opt => opt
                    .MapFrom<DateTimeOffsetToLocalResolver, DateTimeOffset>(
                        s => s.DepartureTime));
    }
}
