using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;

namespace cuqmbr.TravelGuide.Configuration.Logging;

public static class Configuration
{
    public static IServiceCollection ConfigureLogging(
        this IServiceCollection services)
    {
        var configurationOptions = services.BuildServiceProvider().GetService<
            IOptions<cuqmbr.TravelGuide.Application.ConfigurationOptions>>()
            .Value.Logging;

        // TODO: Make enum for available logger types

        services.AddLogging(configuration =>
        {
            configuration.SetMinimumLevel(
                Enum.Parse<LogLevel>(configurationOptions.LogLevel));

            if (configurationOptions.Type.Equals("JsonConsole"))
            {
                configuration.AddJsonConsole(configuration =>
                {
                    if (!String.IsNullOrEmpty(
                            configurationOptions.TimestampFormat))
                    {
                        configuration.TimestampFormat =
                            configurationOptions.TimestampFormat;
                        configuration.UseUtcTimestamp =
                            configurationOptions.UseUtcTimestamp;
                    }
                    configuration.IncludeScopes = true;
                });
            }
            else if (configurationOptions.Type.Equals("SystemdConsole"))
            {
                configuration.AddSystemdConsole(configuration =>
                {
                    if (!String.IsNullOrEmpty(
                            configurationOptions.TimestampFormat))
                    {
                        configuration.TimestampFormat =
                            $"[{configurationOptions.TimestampFormat}] ";
                        configuration.UseUtcTimestamp = true;
                    }
                    configuration.IncludeScopes = true;
                });
            }
            else if (configurationOptions.Type.Equals("SimpleConsole"))
            {
                configuration.AddSimpleConsole(configuration =>
                {
                    // TODO: Uncomment
                    // configuration.SingleLine = true;
                    if (!String.IsNullOrEmpty(
                            configurationOptions.TimestampFormat))
                    {
                        configuration.TimestampFormat =
                            $"[{configurationOptions.TimestampFormat}] ";
                        configuration.UseUtcTimestamp = true;
                    }
                    configuration.IncludeScopes = true;
                });
            }
        });

        return services;
    }
}
