using cuqmbr.TravelGuide.Application.Common.Interfaces.Services;
using System.Globalization;

namespace cuqmbr.TravelGuide.HttpApi.Services;

public sealed class AspNetSessionCultureService : SessionCultureService
{
    private readonly HttpContext _httpContext;
    private const string DefaultCultureId = "en-US";

    public AspNetSessionCultureService(IHttpContextAccessor httpContextAccessor)
    {
        _httpContext = httpContextAccessor.HttpContext!;
    }

    public CultureInfo Culture
    {
        get
        {
            string? cultureId = _httpContext.Request.Headers["Accept-Language"];

            if (cultureId != null)
            {
                cultureId = cultureId.Split(',')[0];
            }
            else
            {
                cultureId = DefaultCultureId;
            }

            CultureInfo result = CultureInfo.GetCultureInfo(DefaultCultureId);

            try
            {
                return CultureInfo.GetCultureInfo(cultureId, true);
            }
            catch (CultureNotFoundException) { }

            CultureInfo.DefaultThreadCurrentCulture = result;
            CultureInfo.DefaultThreadCurrentUICulture = result;

            return result;
        }
    }
}
