using System.IdentityModel.Tokens.Jwt;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Services;
using cuqmbr.TravelGuide.Application.Common.Models;

namespace cuqmbr.TravelGuide.HttpApi.Services;

public sealed class AspNetSessionUserService : SessionUserService
{
    private readonly HttpContext _httpContext;

    public AspNetSessionUserService(IHttpContextAccessor httpContextAccessor)
    {
        _httpContext = httpContextAccessor.HttpContext;
    }

    public int? Id
    {
        get
        {
            var claimValue = _httpContext.User.Claims
                .FirstOrDefault(c => c.Properties
                    .Any(p => p.Value == JwtRegisteredClaimNames.Sub))
                ?.Value;

            var parsed = int.TryParse(claimValue, out var id);

            return parsed ? id : null;
        }
    }

    public Guid? Uuid => Guid.Parse(_httpContext.User.Claims
        .FirstOrDefault(c => c.Properties
            .Any(p => p.Value.Equals("uuid")))
        ?.Value);

    public string? Email => _httpContext.User.Claims
        .FirstOrDefault(c => c.Properties
            .Any(p => p.Value == JwtRegisteredClaimNames.Email))
        ?.Value;

    public ICollection<IdentityRole> Roles => _httpContext.User.Claims
        .Where(c => c.Properties
            .Any(p => p.Value == "roles"))
        .Select(c => IdentityRole.FromName(c.Value))
        .ToArray() ?? default!;

    public string? AccessToken =>
        _httpContext.Request.Cookies["accessToken"] ??
        _httpContext.Request.Headers["Authorization"]
            .ToString()?.Replace("Bearer ", "");
    public string? RefreshToken =>
        _httpContext.Request.Cookies["refreshToken"];
}
