using Microsoft.OpenApi.Models;
using Swashbuckle.AspNetCore.SwaggerGen;

namespace cuqmbr.TravelGuide.HttpApi.Swashbuckle.OperationFilters;

public class AcceptLanguageHeaderOperationFilter : IOperationFilter
{
    public void Apply(OpenApiOperation operation, OperationFilterContext context)
    {
        // TODO: Remove security requirements
        operation.Security ??= new List<OpenApiSecurityRequirement>();

        var acceptLanguage = new OpenApiSecurityScheme
        {
            Reference = new OpenApiReference
            {
                Type = ReferenceType.SecurityScheme,
                Id = "Accept-Language"
            }
        };

        operation.Security.Add(new OpenApiSecurityRequirement
        {
            [acceptLanguage] = new List<string>()
        });

        if (operation.Parameters == null)
            operation.Parameters = new List<OpenApiParameter>();

        operation.Parameters.Add(new OpenApiParameter
        {
            Name = "Accept-Language",
            Description = "Format: <ISO-639 language code>-<ISO-3166 " +
                "country/region code>. " +
                "(https://datatracker.ietf.org/doc/html/rfc4646)",
            In = ParameterLocation.Header,
            Schema = new OpenApiSchema { Type = "String" },
            Required = false
        });
    }
}
