using cuqmbr.TravelGuide.Domain.Entities;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql.Configurations;

public class BaseConfiguration<TEntity> : IEntityTypeConfiguration<TEntity>
    where TEntity : EntityBase
{
    public virtual void Configure(EntityTypeBuilder<TEntity> builder)
    {
        // Set table name for inherited types using type name
        // instead of mapped table name
        var tableName = builder.Metadata.GetTableName();

        builder
            .HasKey(b => b.Id)
            .HasName($"pk_{tableName}");

        builder
            .Property(b => b.Id)
            .HasColumnName("id")
            .HasColumnType("bigint")
            .UseSequence(
                $"{tableName}_" +
                $"{builder.Property(b => b.Id).Metadata.GetColumnName()}_" +
                "sequence");


        builder
            .Property(b => b.Guid)
            .HasColumnName("uuid")
            .HasColumnType("uuid")
            .IsRequired(true);

        builder
            .HasAlternateKey(b => b.Guid)
            .HasName(
                "altk_" +
                $"{tableName}_" +
                $"{builder.Property(b => b.Guid).Metadata.GetColumnName()}");
    }
}
