using cuqmbr.TravelGuide.Domain.Entities;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql.Configurations;

public class CityConfiguration : BaseConfiguration<City>
{
    public override void Configure(EntityTypeBuilder<City> builder)
    {
        builder
            .ToTable("cities");

        base.Configure(builder);


        builder
            .Property(c => c.Name)
            .HasColumnName("name")
            .HasColumnType("varchar(64)")
            .IsRequired(true);


        builder
            .Property(c => c.RegionId)
            .HasColumnName("region_id")
            .HasColumnType("bigint")
            .IsRequired(true);

        builder
            .HasOne(c => c.Region)
            .WithMany(r => r.Cities)
            .HasForeignKey(c => c.RegionId)
            .HasConstraintName(
                "fk_" + 
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(c => c.RegionId).Metadata.GetColumnName()}")
            .OnDelete(DeleteBehavior.Cascade);

        builder
            .HasIndex(c => c.RegionId)
            .HasDatabaseName(
                "ix_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(c => c.RegionId).Metadata.GetColumnName()}");
    }
}
