using cuqmbr.TravelGuide.Domain.Entities;
using cuqmbr.TravelGuide.Domain.Enums;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql.Configurations;

public class VehicleConfiguration : BaseConfiguration<Vehicle>
{
    public override void Configure(EntityTypeBuilder<Vehicle> builder)
    {
        builder
            .Property(a => a.VehicleType)
            .HasColumnName("vehicle_type")
            .HasColumnType("varchar(16)")
            .IsRequired(true);

        builder
            .ToTable(
                "vehicles",
                v => v.HasCheckConstraint(
                    "ck_" +
                    $"{builder.Metadata.GetTableName()}_" +
                    $"{builder.Property(v => v.VehicleType)
                        .Metadata.GetColumnName()}",
                    $"{builder.Property(v => v.VehicleType)
                        .Metadata.GetColumnName()} IN ('{String
                        .Join("', '", VehicleType.Enumerations
                            .Values.Select(v => v.Name))}')"));

        builder
            .ToTable("vehicles")
            .UseTphMappingStrategy()
            .HasDiscriminator(v => v.VehicleType)
            .HasValue<Bus>(VehicleType.Bus)
            .HasValue<Aircraft>(VehicleType.Aircraft)
            .HasValue<Train>(VehicleType.Train);

        base.Configure(builder);


        builder
            .Property(v => v.CompanyId)
            .HasColumnName("company_id")
            .HasColumnType("bigint")
            .IsRequired(true);

        builder
            .HasOne(v => v.Company)
            .WithMany(c => c.Vehicles)
            .HasForeignKey(v => v.CompanyId)
            .HasConstraintName(
                "fk_" + 
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(v => v.CompanyId).Metadata.GetColumnName()}")
            .OnDelete(DeleteBehavior.Cascade);

        builder
            .HasIndex(v => v.CompanyId)
            .HasDatabaseName(
                "ix_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(v => v.CompanyId).Metadata.GetColumnName()}");
    }
}
