﻿using System;
using Microsoft.EntityFrameworkCore.Migrations;

#nullable disable

namespace Persistence.PostgreSql.Migrations
{
    /// <inheritdoc />
    public partial class Add_Vehicle_Enrollment_and_Route_Address_Detail : Migration
    {
        /// <inheritdoc />
        protected override void Up(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.CreateSequence(
                name: "route_address_details_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "vehicle_enrollments_id_sequence",
                schema: "application");

            migrationBuilder.CreateTable(
                name: "vehicle_enrollments",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.vehicle_enrollments_id_sequence')"),
                    departure_time = table.Column<DateTimeOffset>(type: "timestamptz", nullable: false),
                    currency = table.Column<string>(type: "varchar(8)", nullable: false),
                    vehicle_id = table.Column<long>(type: "bigint", nullable: false),
                    route_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_vehicle_enrollments", x => x.id);
                    table.UniqueConstraint("altk_vehicle_enrollments_uuid", x => x.uuid);
                    table.CheckConstraint("ck_vehicle_enrollments_currency", "currency IN ('USD', 'EUR', 'UAH')");
                    table.ForeignKey(
                        name: "fk_vehicle_enrollments_route_id",
                        column: x => x.route_id,
                        principalSchema: "application",
                        principalTable: "routes",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "fk_vehicle_enrollments_vehicle_id",
                        column: x => x.vehicle_id,
                        principalSchema: "application",
                        principalTable: "vehicles",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "route_address_details",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.route_address_details_id_sequence')"),
                    time_to_next_address = table.Column<TimeSpan>(type: "interval", nullable: false),
                    cost_to_next_address = table.Column<decimal>(type: "numeric(24,12)", nullable: false),
                    current_address_stop_time = table.Column<TimeSpan>(type: "interval", nullable: false),
                    vehicle_enrollment_id = table.Column<long>(type: "bigint", nullable: false),
                    route_address_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_route_address_details", x => x.id);
                    table.UniqueConstraint("altk_route_address_details_uuid", x => x.uuid);
                    table.ForeignKey(
                        name: "fk_route_address_details_route_address_id",
                        column: x => x.route_address_id,
                        principalSchema: "application",
                        principalTable: "route_addresses",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "fk_route_address_details_vehicle_enrollment_id",
                        column: x => x.vehicle_enrollment_id,
                        principalSchema: "application",
                        principalTable: "vehicle_enrollments",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateIndex(
                name: "ix_route_address_details_route_address_id",
                schema: "application",
                table: "route_address_details",
                column: "route_address_id");

            migrationBuilder.CreateIndex(
                name: "ix_route_address_details_vehicle_enrollment_id",
                schema: "application",
                table: "route_address_details",
                column: "vehicle_enrollment_id");

            migrationBuilder.CreateIndex(
                name: "ix_vehicle_enrollments_route_id",
                schema: "application",
                table: "vehicle_enrollments",
                column: "route_id");

            migrationBuilder.CreateIndex(
                name: "ix_vehicle_enrollments_vehicle_id",
                schema: "application",
                table: "vehicle_enrollments",
                column: "vehicle_id");
        }

        /// <inheritdoc />
        protected override void Down(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropTable(
                name: "route_address_details",
                schema: "application");

            migrationBuilder.DropTable(
                name: "vehicle_enrollments",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "route_address_details_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "vehicle_enrollments_id_sequence",
                schema: "application");
        }
    }
}
